"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.LruCacheWrapper = void 0;
const DataLoader = require("dataloader");
class LruCacheWrapper {
    constructor(cacheSize = 500) {
        const Lru = require("lru-cache");
        this.cache = new Lru({ max: cacheSize });
    }
    clear() {
        this.cache.reset();
    }
    get(key) {
        return this.cache.get(key) || undefined;
    }
    set(key, value) {
        return this.cache.set(key, value);
    }
    delete(key) {
        if (this.cache.has(key)) {
            this.cache.del(key);
            return true;
        }
        return false;
    }
}
exports.LruCacheWrapper = LruCacheWrapper;
class MongoEntity {
    constructor(connector, collectionName, cache) {
        this.connector = connector;
        this._collectionName = collectionName;
        this._cache = cache ? cache : MongoEntity.DefaultCache;
    }
    get collection() {
        if (!this._collection) {
            this._collection = this.connector.collection(this._collectionName);
        }
        return this._collection;
    }
    createLoader(selectorFunction, options) {
        const opts = options;
        if (opts.cacheMap == "lru") {
            opts.cacheMap = new LruCacheWrapper();
        }
        const loader = new DataLoader((keys) => {
            return Promise.all(keys.map(selectorFunction));
        }, opts);
        if (options) {
            if (options.clearOnInsert) {
                if (!this._insertLoaders) {
                    this._insertLoaders = [];
                }
                this._insertLoaders.push({
                    dataLoader: loader,
                    selectorKeyFn: options.selectorKeyFn,
                });
            }
            if (options.clearOnUpdate) {
                if (!this._updateLoaders) {
                    this._updateLoaders = [];
                }
                if (!options.selectorKeyFn) {
                    throw new Error("You need to provide cache key function to determine when cache needs to be updated");
                }
                this._updateLoaders.push({
                    dataLoader: loader,
                    selectorKeyFn: options.selectorKeyFn,
                });
            }
        }
        return loader;
    }
    clearUpdateCaches(selector) {
        if (this._updateLoaders) {
            this._updateLoaders.forEach((u) => {
                const key = u.selectorKeyFn && u.selectorKeyFn(selector);
                if (key) {
                    u.dataLoader.clear(key);
                }
                else {
                    u.dataLoader.clearAll();
                }
            });
        }
    }
    clearInsertCaches(selector) {
        if (this._insertLoaders) {
            this._insertLoaders.forEach((u) => {
                const key = u.selectorKeyFn && u.selectorKeyFn(selector);
                if (key) {
                    u.dataLoader.clear(key);
                }
                else {
                    u.dataLoader.clearAll();
                }
            });
        }
    }
    assignFilter(object, selector, result, include) {
        return (k) => {
            if (selector[k] != include) {
                throw new Error("You cannot combine include and exclude!");
            }
            if (include) {
                if (object[k] !== undefined) {
                    result[k] = object[k];
                }
            }
            else {
                delete result[k];
            }
        };
    }
    filter(object, selector) {
        let keys = Object.keys(selector);
        if (keys.length == 0) {
            throw new Error("You need to specify the selector!");
        }
        let include = selector[keys[0]];
        let result = include ? {} : Object.assign({}, object);
        let selectorFunction = this.assignFilter(object, selector, result, include);
        keys.forEach(selectorFunction);
        return result;
    }
    async findOneCached(loader, key, selector) {
        const result = await loader.load(key);
        if (result && selector) {
            return this.filter(result, selector);
        }
        else {
            return result;
        }
    }
    async findOneCachedById(id, selector) {
        if (!this._singleLoader) {
            this._singleLoader = this.createLoader((loadId) => {
                return this.collection.findOne({ _id: loadId });
            }, this.addCacheToOptions({ clearOnUpdate: true, clearOnInsert: true, selectorKeyFn: (a) => a._id }));
        }
        return this.findOneCached(this._singleLoader, id, selector);
    }
    async findManyCached(loader, key, selector) {
        if (selector) {
            const result = await loader.load(key);
            return result.map((r) => this.filter(r, selector));
        }
        else {
            return loader.load(key);
        }
    }
    async findAllCached(selector) {
        if (!this._multiLoader) {
            this._multiLoader = this.createLoader(() => this.collection.find().toArray(), this.addCacheToOptions({
                clearOnInsert: true,
                clearOnUpdate: true,
                selectorKeyFn: (a) => null,
            }));
        }
        return this.findManyCached(this._multiLoader, "ALL", selector);
    }
    addCacheToOptions(options) {
        if (options.cacheMap) {
            return options;
        }
        else if (this._cache) {
            options.cacheMap = this._cache;
        }
        else {
            options.cacheMap = "lru";
        }
        return options;
    }
    find(query, options) {
        return this.collection.find(query, options);
    }
    findOne(filter, options) {
        return this.collection.findOne(filter, options);
    }
    insertOne(docs, options) {
        this.clearInsertCaches(docs);
        return this.collection.insertOne(docs, options);
    }
    insertMany(docs, options) {
        this.clearInsertCaches(docs);
        return this.collection.insertMany(docs, options);
    }
    deleteOne(filter, options) {
        this.clearInsertCaches(filter);
        return this.collection.deleteOne(filter, options);
    }
    deleteMany(filter, options) {
        this.clearInsertCaches(filter);
        return this.collection.deleteMany(filter, options);
    }
    dispose() {
        return this.deleteMany({});
    }
    updateOne(filter, update, options) {
        this.clearUpdateCaches(filter);
        return this.collection.updateOne(filter, update, options);
    }
    updateMany(filter, update, options) {
        this.clearUpdateCaches(filter);
        return this.collection.updateMany(filter, update, options);
    }
}
exports.default = MongoEntity;
