"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.stopServer = exports.stopDatabase = exports.getConnector = exports.itWithServer = exports.withServer = exports.itWithContext = exports.withContext = exports.itWithEntity = exports.withEntity = exports.getDb = exports.config = void 0;
const mongodb_1 = require("mongodb");
const mongo_entity_1 = require("./mongo_entity");
const mongodb_memory_server_1 = require("mongodb-memory-server");
let customGlobal = global;
customGlobal.__mongoConnectorConfig = {
    db: null,
    client: null,
    initContext: null,
    testServer: null,
    verbose: false,
};
let glob = customGlobal.__mongoConnectorConfig;
function config({ verbose, initContext: ic, testServer: ts, }) {
    glob.initContext = ic || glob.initContext;
    glob.testServer = ts || glob.testServer;
    glob.verbose = verbose;
}
exports.config = config;
async function getDb() {
    const mongod = new mongodb_memory_server_1.MongoMemoryServer();
    if (glob.db) {
        return glob.db;
    }
    const uri = await mongod.getUri();
    const dbName = await mongod.getDbName();
    console.log("Connecting to: " + uri);
    let client = await mongodb_1.MongoClient.connect(uri);
    glob.db = await client.db(dbName);
    glob.client = client;
    console.log("Connected!");
    return glob.db;
}
exports.getDb = getDb;
async function withEntity(test, options) {
    const connector = await getConnector();
    let entities = [];
    if (options && options.entities) {
        for (let i = 0; i < options.entities.length; i++) {
            const option = options.entities[i];
            const name = option.name;
            const type = option.type ? option.type : mongo_entity_1.default;
            let entity = new type(connector, name ? name : `test_${i}`);
            entities.push(entity);
            if (options.entities[i].data) {
                for (let data of options.entities[i].data) {
                    await entity.insertOne(data);
                }
            }
        }
    }
    else {
        entities.push(new mongo_entity_1.default(connector, "test"));
    }
    try {
        await test(...entities);
    }
    catch (ex) {
        throw ex;
    }
    finally {
        await entities.forEach((e) => e.deleteMany({}));
    }
}
exports.withEntity = withEntity;
function itWithEntity(name, func, options) {
    it(name, async function () {
        await withEntity(async (context) => {
            await func(context);
        }, options);
    });
}
exports.itWithEntity = itWithEntity;
const fakeConnector = {
    collection() {
        return {
            deleteMany() { },
            insert() { },
            find() { },
            findOne() { },
        };
    },
};
async function withContext(test, initContextFn, disconnected = false) {
    const connector = disconnected ? fakeConnector : await getConnector();
    glob.initContext = initContextFn || glob.initContext;
    if (!glob.initContext) {
        throw new Error("No initContext provided, please pass as a parameter or use global config");
    }
    const context = glob.initContext(connector);
    try {
        await test(context);
    }
    catch (ex) {
        throw ex;
    }
    finally {
        for (let key of Object.keys(context)) {
            if (context[key] && context[key].dispose) {
                await context[key].dispose({}, true);
            }
        }
    }
}
exports.withContext = withContext;
function itWithContext(name, func, initContextFn, disconnected) {
    it(name, async function () {
        await withContext(async (context) => {
            await func(context);
        }, initContextFn, disconnected);
    });
}
exports.itWithContext = itWithContext;
async function withServer(test, server) {
    if (server) {
        glob.testServer = server;
    }
    else {
        server = glob.testServer;
    }
    if (!server) {
        throw new Error("No server provided, please pass as a parameter or use global config");
    }
    if (!server.started) {
        await server.startTest();
    }
    try {
        await test(server);
    }
    catch (ex) {
        throw ex;
    }
    finally {
        for (let key of Object.keys(server.context)) {
            if (server.context[key] && server.context[key].dispose) {
                await server.context[key].dispose({}, true);
            }
        }
    }
}
exports.withServer = withServer;
function itWithServer(name, func, server) {
    it(name, async function () {
        await withServer(async (context) => {
            await func(context);
        }, server);
    });
}
exports.itWithServer = itWithServer;
async function getConnector() {
    let myDb = await getDb();
    return {
        collection(name) {
            return glob.db.collection(name);
        },
    };
}
exports.getConnector = getConnector;
async function stopDatabase() {
    let myDb = glob.db;
    let client = glob.client;
    glob.db = null;
    if (myDb) {
        await myDb.dropDatabase();
        await client.close();
    }
}
exports.stopDatabase = stopDatabase;
function stopServer() {
    if (!glob.testServer) {
        console.error("No server to stop!");
    }
    console.log("Stopping test server");
    glob.testServer.stopTest();
}
exports.stopServer = stopServer;
