"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const assert = require("power-assert");
const sinon = require("sinon");
const proxyquire = require("proxyquire");
const testing_1 = require("../testing");
const mongo_entity_1 = require("../mongo_entity");
const host = process.env.MONGODB_HOST || "127.0.0.1";
const port = process.env.MONGODB_PORT || 27017;
describe("entity", () => {
    it("contains connector and collection name", async () => {
        var connector = await testing_1.getConnector();
        var collectionName = "name";
        var entity = new mongo_entity_1.default(connector, collectionName);
        assert.equal(entity.collection.collectionName, collectionName);
        assert.equal(entity.connector, connector);
        assert.notEqual(entity.collection, null);
    });
    it("is possible to initialise entity with custom cache", function () {
        const cache = { local: "" };
        const entity = new mongo_entity_1.default(null, "name", cache);
        assert.equal(entity["_cache"], cache);
        const globalCache = { global: "1" };
        mongo_entity_1.default.DefaultCache = globalCache;
        const otherEntity = new mongo_entity_1.default(null, "name");
        assert.equal(otherEntity["_cache"], globalCache);
        mongo_entity_1.default.DefaultCache = null;
    });
    describe("find", () => {
        it("can find a multiple record", async () => {
            await testing_1.withEntity((entity) => {
                const find = sinon.stub(entity.collection, "find");
                const selector = {};
                const options = {};
                entity.find(selector, options);
                sinon.assert.calledWith(find, selector, options);
            });
        });
    });
    describe("findOne", () => {
        it("can find a single record", async () => {
            await testing_1.withEntity((entity) => {
                const find = sinon.stub(entity.collection, "findOne");
                const selector = {};
                const options = {};
                entity.findOne(selector, options);
                sinon.assert.calledWith(find, selector, options);
            });
        });
    });
    describe("findOneCachedById", () => {
        testing_1.itWithEntity("returns an item even when it was not previously found, but it wont search twice", async (entity) => {
            const findSpy = sinon.spy(entity.collection, "findOne");
            let result = await entity.findOneCachedById("1");
            assert.equal(result, null);
            sinon.assert.calledOnce(findSpy);
            result = await entity.findOneCachedById("1");
            assert.equal(result, null);
            sinon.assert.calledOnce(findSpy);
            const element = { _id: "1" };
            await entity.insertOne(element);
            result = await entity.findOneCachedById("1");
            assert.deepEqual(result, element);
            sinon.assert.calledTwice(findSpy);
        });
        testing_1.itWithEntity("can find and cache results of finding a single item", async (entity) => {
            const find = sinon
                .stub(entity.collection, "findOne")
                .returns({ _id: "1" });
            let result = await entity.findOne({ _id: "1" });
            result = await entity.findOne({ _id: "1" });
            assert.deepEqual(result, { _id: "1" });
            assert(find.calledTwice);
            find.resetHistory();
            result = await entity.findOneCachedById("1");
            result = await entity.findOneCachedById("1");
            assert(find.calledOnce);
        }, {
            entities: [
                {
                    data: [{ _id: "1" }],
                },
            ],
        });
        testing_1.itWithEntity("clears the related cache when a document is updated", async function (entity) {
            const findSpy = sinon.spy(entity.collection, "findOne");
            const cacheSpy = sinon.spy(entity, "clearUpdateCaches");
            await entity.insertOne({ _id: "1", file: "foo" });
            await entity.insertOne({ _id: "2", file: "bar" });
            let foo = await entity.findOneCachedById("1");
            let bar = await entity.findOneCachedById("2");
            const selector = { _id: "1" };
            await entity.updateOne(selector, { $set: { file: "boo" } });
            assert(cacheSpy.calledWith(selector));
            findSpy.resetHistory();
            foo = await entity.findOneCachedById("1");
            assert.deepEqual(foo, { _id: "1", file: "boo" });
            assert(findSpy.calledOnce);
            findSpy.resetHistory();
            bar = await entity.findOneCachedById("2");
            assert(findSpy.notCalled);
        });
        testing_1.itWithEntity("clears all caches when a document is updated and selector is unknown", async function (entity) {
            let foo = await entity.findOneCachedById("1");
            let bar = await entity.findOneCachedById("2");
            entity.updateOne({ file: "foo" }, { $set: { file: "boo" } });
            const findSpy = sinon.spy(entity.collection, "findOne");
            foo = await entity.findOneCachedById("1");
            bar = await entity.findOneCachedById("2");
            assert(findSpy.calledTwice);
        });
        testing_1.itWithEntity("findOneCached can filter returned results", async (entity) => {
            await entity.insertOne({ _id: "00", a: "1", b: "2", c: "3" });
            let result = await entity.findOneCachedById("00", { b: 1 });
            assert.deepEqual(result, { b: "2" });
        });
        testing_1.itWithEntity("findOneCached returns null if entity does not exists", async (entity) => {
            let result = await entity.findOneCachedById("0");
            assert.deepEqual(result, null);
            result = await entity.findOneCachedById("0", { field: 1 });
            assert.deepEqual(result, null);
        });
    });
    describe("updates", () => {
        testing_1.itWithEntity("one", async function (entity) {
            const stub = sinon.stub(entity.collection, "updateOne");
            const selector = {};
            const modifier = {};
            const properties = {};
            entity.updateOne(selector, modifier, properties);
            sinon.assert.calledWith(stub, selector, modifier, properties);
        });
        testing_1.itWithEntity("many", async function (entity) {
            const stub = sinon.stub(entity.collection, "updateMany");
            const selector = {};
            const modifier = {};
            const properties = {};
            entity.updateMany(selector, modifier, properties);
            sinon.assert.calledWith(stub, selector, modifier, properties);
        });
    });
    describe("addCacheToOptions", () => {
        it("adds entitys cache to options", function () {
            const cache = { g: 1 };
            const entity = new mongo_entity_1.default(null, null, cache);
            const options = {};
            const res1 = entity.addCacheToOptions(options);
            assert.deepEqual(res1, { cacheMap: cache });
            const optionsWithCache = { cacheMap: {} };
            const res2 = entity.addCacheToOptions(optionsWithCache);
            assert.deepEqual(res2, optionsWithCache);
        });
    });
    describe("filter", () => {
        it("can filter objects properties", function () {
            const entity = new mongo_entity_1.default(null, null);
            const obj = { a: 1, b: 2, c: 3 };
            const resultWith = entity.filter(obj, { a: 1, c: 1 });
            assert.deepEqual(resultWith, { a: 1, c: 3 });
            const resultWithout = entity.filter(obj, { a: 0, c: 0 });
            assert.deepEqual(resultWithout, { b: 2 });
        });
        it("does not include undefined elements", function () {
            const entity = new mongo_entity_1.default(null, null);
            const obj = { a: 1, b: 2, c: 3 };
            const resultWith = entity.filter(obj, { a: 1, c: 1, d: 1 });
            assert.deepEqual(resultWith, { a: 1, c: 3 });
        });
        it("throws error when no filter is specified", function () {
            const entity = new mongo_entity_1.default(null, null);
            const obj = {};
            assert.throws(() => entity.filter(obj, {}), /You need to specify the selector!/);
        });
        it("throws error when you mix include exclude", function () {
            const entity = new mongo_entity_1.default(null, null);
            const obj = {};
            assert.throws(() => entity.filter(obj, { a: 1, b: 0 }), /You cannot combine include and exclude!/);
        });
    });
    describe("findManyCached", () => {
        it("can find and cache results of finding multiple items", async () => {
            await testing_1.withEntity(async (entity) => {
                entity.collection.insertOne({ _id: "1" });
                const find = sinon.spy(entity.collection, "find");
                let result = await entity.findAllCached();
                result = await entity.findAllCached();
                assert.deepEqual(result, [{ _id: "1" }]);
                assert(find.calledOnce);
            });
        });
        it("findManyCached can filter returned results", async () => {
            await testing_1.withEntity(async (entity) => {
                await entity.insertMany([
                    { _id: "00", a: "1", b: "2", c: "3" },
                    { _id: "01", a: "1", b: "2", c: "3" },
                ]);
                let result = await entity.findAllCached({ a: 1, c: 1 });
                assert.deepEqual(result, [
                    { a: "1", c: "3" },
                    { a: "1", c: "3" },
                ]);
            });
        });
        it("clears the cache when a new document is inserted", async function () {
            await testing_1.withEntity(async (entity) => {
                const find = sinon.spy(entity.collection, "find");
                entity.insertOne({ _id: "1" });
                let result = await entity.findAllCached();
                await entity.insertOne({ _id: "2" });
                result = await entity.findAllCached();
                assert(find.calledTwice);
                assert.deepEqual(result, [{ _id: "1" }, { _id: "2" }]);
            });
        });
    });
    describe("delete", () => {
        it("can delete a single document", async () => {
            await testing_1.withEntity(async (entity) => {
                await entity.insertOne({ _id: "00", a: "1", b: "2", c: "3" });
                await entity.insertOne({ _id: "01", a: "1", b: "2", c: "3" });
                let result = await entity.findAllCached();
                assert.equal(result.length, 2);
                await entity.deleteOne({ _id: "00" });
                result = await entity.findAllCached();
                assert.equal(result.length, 1);
            });
        });
        it("can delete multiple documents", async function () {
            await testing_1.withEntity(async (entity) => {
                await entity.insertOne({ _id: "00", a: "1", b: "2", c: "3" });
                await entity.insertOne({ _id: "01", a: "1", b: "2", c: "3" });
                await entity.deleteMany({});
                const result = await entity.findAllCached();
                assert.equal(result.length, 0);
            });
        });
    });
    describe("dispose", () => {
        it("deletes all records", function () {
            const entity = new mongo_entity_1.default(null, "name");
            const stub = sinon.stub(entity, "deleteMany");
            entity.dispose();
            sinon.assert.calledWith(stub, {});
        });
    });
    describe("testing", () => {
        it("can execute test with multiple entities", async function () {
            await testing_1.withEntity((e1, e2, e3) => {
                assert(e1);
                assert.equal(e1.collection.collectionName, "e1");
                assert(e2);
                assert.equal(e2.collection.collectionName, "e2");
                assert(e3);
                assert.equal(e3.collection.collectionName, "e3");
            }, { entities: [{ name: "e1" }, { name: "e2" }, { name: "e3" }] });
        });
    });
    describe("custom loaders", () => {
        it("throws error when selectorKeyFunction is not specified with updates", async () => {
            await testing_1.withEntity((entity) => {
                assert.throws(() => entity.createLoader((() => { }), { clearOnUpdate: true }), /You need to provide cache key function to determine when cache needs to be updated/);
            });
        });
        it("can create a new data-loader with a custom cache", async function () {
            await testing_1.withEntity(async (entity) => {
                const records = [
                    { _id: 1, name: "A" },
                    { _id: 2, name: "B" },
                    { _id: 3, name: "C" },
                ];
                entity.insertMany(records);
                const loader = entity.createLoader((name) => {
                    return entity.collection.findOne({ name });
                }, {
                    cacheMap: new mongo_entity_1.LruCacheWrapper(2),
                    clearOnInsert: true,
                    clearOnUpdate: true,
                    selectorKeyFn: (a) => a.name,
                });
                const spy = sinon.spy(entity.collection, "findOne");
                await entity.findOneCached(loader, "A");
                await entity.findOneCached(loader, "A");
                await entity.findOneCached(loader, "B");
                await entity.findOneCached(loader, "B");
                await entity.findOneCached(loader, "A");
                sinon.assert.calledTwice(spy);
                spy.resetHistory();
                await entity.updateOne({ _id: 1 }, { $set: { name: "D" } });
                const a = await entity.findOneCached(loader, "A");
                assert.equal(a, null);
                sinon.assert.calledOnce(spy);
                spy.resetHistory();
                await entity.findOneCached(loader, "C");
                await entity.findOneCached(loader, "A");
                sinon.assert.calledOnce(spy);
                spy.resetHistory();
                await entity.findOneCached(loader, "B");
                sinon.assert.calledOnce(spy);
                spy.resetHistory();
                await entity.findOneCached(loader, "C");
                sinon.assert.calledOnce(spy);
            });
        });
    });
    describe("LruCacheWrapper", () => {
        it("maps lru-cache functions", function () {
            function cacheStub() {
                return {
                    get: sinon.stub(),
                    set: sinon.stub(),
                    reset: sinon.stub(),
                    has: sinon.stub(),
                    del: sinon.stub(),
                };
            }
            const { LruCacheWrapper } = proxyquire("../mongo_entity", {
                "lru-cache": cacheStub,
            });
            const cacheWrapper = new LruCacheWrapper();
            cacheWrapper.clear();
            sinon.assert.calledOnce(cacheWrapper.cache.reset);
            cacheWrapper.get();
            sinon.assert.calledOnce(cacheWrapper.cache.get);
            cacheWrapper.set();
            sinon.assert.calledOnce(cacheWrapper.cache.set);
            cacheWrapper.cache.has.returns(true);
            const positiveResult = cacheWrapper.delete("1");
            assert(positiveResult);
            sinon.assert.calledWith(cacheWrapper.cache.del, "1");
            cacheWrapper.cache.has.returns(false);
            const negativeResult = cacheWrapper.delete("1");
            assert.equal(negativeResult, false);
        });
    });
});
