"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const sinon = require("sinon");
const mongo_entity_1 = require("../mongo_entity");
const testing_1 = require("../testing");
const assert = require("power-assert");
const proxyquire = require("proxyquire");
class CutomEntity extends mongo_entity_1.default {
    constructor() {
        super(...arguments);
        this.name = "customEntity";
    }
}
describe("Testing Helpers", () => {
    describe("withEntity", () => {
        it("cleans up after error has been thrown", async () => {
            let deleteSpy = null;
            try {
                await testing_1.withEntity(async (entity) => {
                    deleteSpy = sinon.spy(entity, "deleteMany");
                    throw new Error("Error");
                });
            }
            catch (ex) {
            }
            sinon.assert.calledOnce(deleteSpy);
        });
        it("can create custom entities", async () => {
            await testing_1.withEntity(async (entity) => {
                assert.equal(entity.name, "customEntity");
            }, { entities: [{ type: CutomEntity }] });
        });
        it("can create multiple entities", async () => {
            await testing_1.withEntity(async (one, two) => {
                assert.equal(one.collection.collectionName, "One");
                assert.equal(two.collection.collectionName, "Two");
            }, { entities: [{ name: "One" }, { name: "Two" }] });
        });
    });
    describe("withContext", () => {
        it("throws error when no initContext is specified", async () => {
            let message = "";
            try {
                await testing_1.withContext(async (context) => { });
            }
            catch (ex) {
                message = ex.message;
            }
            assert.equal(message, "No initContext provided, please pass as a parameter or use global config");
        });
        it("initializes context with connection, executes tests and cleans up afterwards", async function () {
            function initContext(conn) {
                return {
                    entity: new mongo_entity_1.default(conn, "col"),
                    other: {},
                };
            }
            let inContext = null;
            let deleteSpy = null;
            await testing_1.withContext(async (context) => {
                assert(context.entity);
                deleteSpy = sinon.spy(context.entity, "deleteMany");
                inContext = context;
            }, initContext);
            sinon.assert.calledOnce(deleteSpy);
        });
        it("throws exception but also cleans up", async function () {
            function initContext(conn) {
                return {
                    entity: new mongo_entity_1.default(conn, "col"),
                };
            }
            let inContext = null;
            let deleteSpy = null;
            try {
                await testing_1.withContext(async (context) => {
                    deleteSpy = sinon.spy(context.entity, "deleteMany");
                    inContext = context;
                    throw new Error("Error");
                }, initContext);
            }
            catch (ex) {
                assert.equal(ex.message, "Error");
            }
            sinon.assert.calledOnce(deleteSpy);
        });
        it("can run from the globally declared contextFunction", async () => {
            const initContext = sinon.stub().returns({});
            testing_1.config({ initContext });
            await testing_1.withContext(() => { });
            sinon.assert.calledOnce(initContext);
        });
        it("can run in the disconnected state", async () => {
            const MongoStub = {
                MongoClient: sinon.stub(),
            };
            const { withContext, config } = proxyquire("../testing", {
                mongodb: MongoStub,
            });
            const initContext = (connector) => {
                assert(connector.collection());
                return {};
            };
            config({ initContext });
            await withContext((context) => { }, null, true);
            sinon.assert.notCalled(MongoStub.MongoClient);
        });
    });
    describe("withServer", () => {
        let server;
        beforeEach(() => {
            server = {
                startTest: sinon.stub(),
                stopTest: sinon.stub(),
                context: [{ dispose: sinon.stub() }],
            };
        });
        it("throws error when no server is specified", async () => {
            let message = "";
            try {
                await testing_1.withServer(async (context) => { });
            }
            catch (ex) {
                message = ex.message;
            }
            assert.equal(message, "No server provided, please pass as a parameter or use global config");
        });
        it("starts a new server when server is not started", async () => {
            await testing_1.withServer((context) => { }, server);
            sinon.assert.calledOnce(server.startTest);
        });
        it("cleans up resources even with exception", async () => {
            try {
                await testing_1.withServer((context) => {
                    throw "";
                }, server);
            }
            catch (ex) {
            }
            sinon.assert.calledOnce(server.context[0].dispose);
        });
    });
    describe("Mocha helpers", () => {
        describe("itWithContext", () => {
            testing_1.itWithContext("runs", (context) => {
                assert(true);
            });
        });
        describe("itWithEntity", () => {
            testing_1.itWithEntity("runs", (context) => {
                assert(true);
            });
        });
        describe("itWithServer", () => {
            const server = {
                startTest: sinon.stub(),
                context: [],
            };
            testing_1.itWithServer("runs", (context) => {
                assert(true);
            }, server);
        });
    });
});
